from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.technician_config_repository_abc import TechnicianConfigRepositoryABC
from bot_data.model.technician_config import TechnicianConfig
from bot_data.model.user_role_enum import UserRoleEnum
from bot_graphql.abc.query_abc import QueryABC


class TechnicianConfigMutation(QueryABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
        servers: ServerRepositoryABC,
        technician_configs: TechnicianConfigRepositoryABC,
        db: DatabaseContextABC,
    ):
        QueryABC.__init__(self, "TechnicianConfigMutation")

        self._bot = bot
        self._servers = servers
        self._technician_configs = technician_configs
        self._db = db

        self.set_field("createTechnicianConfig", self.resolve_create_TechnicianConfig)
        self.set_field("updateTechnicianConfig", self.resolve_update_TechnicianConfig)
        self.set_field("deleteTechnicianConfig", self.resolve_delete_TechnicianConfig)

    def resolve_create_technician_config(self, *_, input: dict):
        if self._technician_configs.does_technician_config_exists():
            return None

        technician_config = TechnicianConfig(
            input["helpCommandReferenceUrl"],
            input["waitForRestart"],
            input["waitForShutdown"],
            input["cacheMaxMessages"],
            input["pingURLs"],
            input["technicianIds"],
        )
        self._technician_configs.add_technician_config(technician_config)
        self._db.save_changes()

        return self._technician_configs.get_technician_config()

    def resolve_update_technician_config(self, *_, input: dict):
        technician_config = self._technician_configs.get_technician_config()
        self._can_user_mutate_data(technician_config, UserRoleEnum.admin)

        technician_config.name = input["name"] if "name" in input else technician_config.name

        self._technician_configs.update_TechnicianConfig(technician_config)
        self._db.save_changes()

        technician_config = self._technician_configs.get_TechnicianConfig_by_id(input["id"])
        return technician_config

    def resolve_delete_technician_config(self, *_, id: int):
        technician_config = self._technician_configs.get_TechnicianConfig_by_id(id)
        self._can_user_mutate_data(technician_config.server, UserRoleEnum.admin)

        self._technician_configs.delete_TechnicianConfig(technician_config)
        self._db.save_changes()

        return technician_config
